@@ -45,38 +45,44 @@ import logging
 import whoosh
 import whoosh.index
 import whoosh.qparser
+from whoosh.fields import Schema, FieldType
+from whoosh.writing import IndexWriter
 
 import papis.config
+import papis.strings
 import papis.document
 import papis.database.base
 import papis.database.cache
+from papis.utils import get_cache_home, get_folders, folders_to_documents
+
+from typing import List, Dict, Optional, Any, KeysView
 
 
 class Database(papis.database.base.Database):
 
-    def __init__(self, library=None):
+    def __init__(self, library: Optional[papis.library.Library] = None):
         papis.database.base.Database.__init__(self, library)
         self.logger = logging.getLogger('db:whoosh')
+        self.cache_dir = os.path.join(get_cache_home(), 'database', 'whoosh')
+        self.index_dir = os.path.expanduser(
+            os.path.join(
+                self.cache_dir,
+                papis.database.cache.get_cache_file_name(
+                    self.lib.path_format()
+                )))  # type: str
+
         self.initialize()
 
-    def get_backend_name(self):
+    def get_backend_name(self) -> str:
         return 'whoosh'
 
-    def clear(self):
+    def clear(self) -> None:
         import shutil
         if self.index_exists():
             self.logger.warning('Clearing the database')
-            shutil.rmtree(self.get_index_dir())
-        else:
-            self.logger.warning(
-                'Trying to clear database, but no database found'
-            )
+            shutil.rmtree(self.index_dir)
 
-#   TODO
-    def match(self, document, query_string):
-        pass
-
-    def add(self, document):
+    def add(self, document: papis.document.Document) -> None:
         schema_keys = self.get_schema_init_fields().keys()
         self.logger.debug("adding document")
         writer = self.get_writer()
@@ -84,29 +90,29 @@ class Database(papis.database.base.Database):
         self.logger.debug("commiting document..")
         writer.commit()
 
-    def update(self, document):
+    def update(self, document: papis.document.Document) -> None:
         """As it says in the docs, just delete the document and add it again
         """
         self.delete(document)
         self.add(document)
 
-    def delete(self, document):
+    def delete(self, document: papis.document.Document) -> None:
         writer = self.get_writer()
         self.logger.debug("deleting document..")
         writer.delete_by_term(
             self.get_id_key(),
-            self.get_id_value(document)
-        )
+            self.get_id_value(document))
         self.logger.debug("commiting deletion..")
         writer.commit()
 
-    def query_dict(self, dictionary):
+    def query_dict(
+            self, dictionary: Dict[str, str]) -> List[papis.document.Document]:
         query_string = " AND ".join(
-            ["{}:\"{}\" ".format(key, val) for key, val in dictionary.items()]
-        )
+            ["{}:\"{}\" ".format(key, val)
+                for key, val in dictionary.items()])
         return self.query(query_string)
 
-    def query(self, query_string):
+    def query(self, query_string: str) -> List[papis.document.Document]:
         self.logger.debug('Query string %s' % query_string)
         index = self.get_index()
         qp = whoosh.qparser.MultifieldParser(
@@ -120,17 +126,16 @@ class Database(papis.database.base.Database):
             self.logger.debug(results)
             documents = [
                 papis.document.from_folder(r.get(self.get_id_key()))
-                for r in results
-            ]
+                for r in results]
         return documents
 
-    def get_all_query_string(self):
+    def get_all_query_string(self) -> str:
         return '*'
 
-    def get_all_documents(self):
+    def get_all_documents(self) -> List[papis.document.Document]:
         return self.query(self.get_all_query_string())
 
-    def get_id_key(self):
+    def get_id_key(self) -> str:
         """Get the unique key identifier name of the documents in the database
 
         :returns: key identifier
@@ -138,7 +143,7 @@ class Database(papis.database.base.Database):
         """
         return 'whoosh_id_'
 
-    def get_id_value(self, document):
+    def get_id_value(self, document: papis.document.Document) -> str:
         """Get the value that is stored in the unique key identifier
         of the documents in the database. In the case of papis this is
         just the path of the documents.
@@ -148,25 +153,32 @@ class Database(papis.database.base.Database):
         :returns: Path for the document
         :rtype:  str
         """
-        return document.get_main_folder()
+        _folder = document.get_main_folder()
+        if _folder is None:
+            raise Exception(papis.strings.no_folder_attached_to_document)
+        else:
+            return _folder
 
-    def create_index(self):
+    def create_index(self) -> None:
         """Create a brand new index, notice that if an index already
         exists it will delete it and create a new one.
         """
         self.logger.debug('Creating index...')
-        index_dir = self.get_index_dir()
-        if not os.path.exists(index_dir):
-            self.logger.debug('Creating dir %s' % index_dir)
-            os.makedirs(index_dir)
-        whoosh.index.create_in(self.get_index_dir(), self.create_schema())
+        if not os.path.exists(self.index_dir):
+            self.logger.debug('Creating dir %s' % self.index_dir)
+            os.makedirs(self.index_dir)
+        whoosh.index.create_in(self.index_dir, self.create_schema())
 
-    def index_exists(self):
-        """Check if index already exists in get_index_dir()
+    def index_exists(self) -> Any:
+        """Check if index already exists in index_dir()
         """
-        return whoosh.index.exists_in(self.get_index_dir())
+        return whoosh.index.exists_in(self.index_dir)
 
-    def add_document_with_writer(self, document, writer, schema_keys):
+    def add_document_with_writer(
+            self,
+            document: papis.document.Document,
+            writer: IndexWriter,
+            schema_keys: KeysView[str]) -> None:
         """Helper function that takes a writer and a dictionary
         containing the keys of the schema and adds the document to the writer.
         Notice that this function does only two things, creating a suitable
@@ -177,7 +189,7 @@ class Database(papis.database.base.Database):
         :param document: Papis document
         :type  document: papis.document.Document
         :param writer: Whoosh writer
-        :type  writer: whoosh.writer
+        :type  writer: whoosh.writing.IndexWriter
         :param schema_keys: Dictionary containing the defining keys of the
             database Schema
         :type  schema_keys: dict
@@ -192,7 +204,7 @@ class Database(papis.database.base.Database):
         doc_d[self.get_id_key()] = self.get_id_value(document)
         writer.add_document(**doc_d)
 
-    def do_indexing(self):
+    def do_indexing(self) -> None:
         """This function initializes the database. Basically it goes through
         all folders from the library (that contain an `info.yaml` file)
         and adds the documents to the database index. This function is
@@ -200,34 +212,70 @@ class Database(papis.database.base.Database):
         at the time of building a brand new index.
         """
         self.logger.debug('Indexing the library, this might take a while...')
-        folders = papis.utils.get_folders(self.get_dir())
-        documents = papis.database.cache.folders_to_documents(folders)
+        folders = sum(
+                [get_folders(d)
+                    for d in self.get_dirs()], [])  # type: List[str]
+        documents = folders_to_documents(folders)
         schema_keys = self.get_schema_init_fields().keys()
         writer = self.get_writer()
         for doc in documents:
             self.add_document_with_writer(doc, writer, schema_keys)
         writer.commit()
 
-    def initialize(self):
+    def initialize(self) -> None:
         """Function to be called everytime a database object is created.
         It checks if an index exists, if not, it creates one and
         indexes the library.
+
+        If the schema fields have been changed, it updates the database.
         """
         if self.index_exists():
-            self.logger.debug('Initialized index found for library')
-            return True
+            user_fields = self.get_schema_init_fields()
+            db_fields = self.get_schema()
+
+            user_field_names = sorted(list(user_fields))
+            db_field_names = sorted(db_fields.names())
+
+            # If the user fields and the fields in the DB
+            # aren't the same, then we have to rebuild the
+            # database.
+            if user_field_names != db_field_names:
+                self.rebuild()
+                self.logger.debug("Rebuilt database because field names"
+                                  "don't match")
+            else:
+                # Otherwise, verify that the fields are
+                # all the same and rebuild if any have
+                # changed at all.
+                rebuilt_db = False
+                for field in user_field_names:
+                    if user_fields[field] != db_fields[field]:
+                        self.rebuild()
+                        self.logger.debug("Rebuilt DB because field types"
+                                          " don't match")
+                        rebuilt_db = True
+                        break
+
+                if not rebuilt_db:
+                    self.logger.debug('Initialized index found for library')
+                    return
         self.create_index()
         self.do_indexing()
 
-    def get_index(self):
+    def rebuild(self) -> None:
+        self.clear()
+        self.create_index()
+        self.do_indexing()
+
+    def get_index(self) -> whoosh.index.Index:
         """Gets the index for the current library
 
         :returns: Index
         :rtype:  whoosh.index
         """
-        return whoosh.index.open_dir(self.get_index_dir())
+        return whoosh.index.open_dir(self.index_dir)
 
-    def get_writer(self):
+    def get_writer(self) -> IndexWriter:
         """Gets the writer for the current library
 
         :returns: Writer
@@ -235,7 +283,7 @@ class Database(papis.database.base.Database):
         """
         return self.get_index().writer()
 
-    def get_schema(self):
+    def get_schema(self) -> Schema:
         """Gets current schema
 
         :returns: Whoosch Schema
@@ -243,7 +291,7 @@ class Database(papis.database.base.Database):
         """
         return self.get_index().schema
 
-    def create_schema(self):
+    def create_schema(self) -> Schema:
         """Creates and returns whoosh schema to be applied to the library
 
         :returns: Whoosch Schema
@@ -255,48 +303,20 @@ class Database(papis.database.base.Database):
         schema = Schema(**fields)
         return schema
 
-    def get_schema_init_fields(self):
+    def get_schema_init_fields(self) -> Dict[str, FieldType]:
         """Returns the arguments to be passed to the whoosh schema
         object instantiation found in the method `get_schema`.
         """
-        from whoosh.fields import TEXT, ID, KEYWORD, STORED
+        # This we need for the eval code beneath
+        from whoosh.fields import TEXT, ID, KEYWORD, STORED  # noqa: F401
         # This part is non-negotiable
         fields = {self.get_id_key(): ID(stored=True, unique=True)}
+        # TODO: this is a security risk, find a way to fix it
         user_prototype = eval(
-            papis.config.get('whoosh-schema-prototype')
-        )
+            papis.config.getstring('whoosh-schema-prototype'))  # KeysView[str]
         fields.update(user_prototype)
-        fields_list = eval(papis.config.get('whoosh-schema-fields'))
-        if not isinstance(fields_list, list):
-            raise Exception('whoosh-schema-fields should be a python list')
+        fields_list = papis.config.getlist('whoosh-schema-fields')
         for field in fields_list:
             fields.update({field: TEXT(stored=True)})
         # self.logger.debug('Schema prototype: {}'.format(fields))
         return fields
-
-    def get_cache_dir(self):
-        """Get general directory to store whoosh indexes.
-
-        :returns: Full path to whoosh cache home directory
-        :rtype:  str
-        """
-        path = os.path.join(
-            papis.database.cache.get_cache_home(),
-            'whoosh'
-        )
-        # self.logger.debug('Cache dir %s' % path)
-        return path
-
-    def get_index_dir(self):
-        """Get the directory inside `get_cache_dir` to store the index.
-        :returns: Full path to index dir
-        :rtype:  str
-        """
-        path = os.path.expanduser(
-            os.path.join(
-                self.get_cache_dir(),
-                papis.database.cache.get_name(self.get_dir())
-            )
-        )
-        # self.logger.debug('Index dir %s' % path)
-        return path
