@@ -1,5 +1,8 @@
 import papis.config
+import papis.document
 import logging
+from typing import Optional, List, Any, Callable
+MATCHER_TYPE = Callable[[papis.document.Document, str, Optional[str]], Any]
 
 
 class DocMatcher(object):
@@ -17,20 +20,20 @@ class DocMatcher(object):
     via the `return_if_match` method, which is used to parallelize the
     matching.
     """
-    search = ""
-    parsed_search = None
-    doc_format = '{' + papis.config.get('format-doc-name') + '[DOC_KEY]}'
+    search = ""  # type: str
+    parsed_search = []  # type: List[Any]
+    doc_format = '{%s[DOC_KEY]}' % (papis.config.getstring('format-doc-name'))
     logger = logging.getLogger('DocMatcher')
-    matcher = None
+    matcher = None  # type: Optional[MATCHER_TYPE]
 
     @classmethod
-    def return_if_match(cls, doc):
+    def return_if_match(
+            cls,
+            doc: papis.document.Document) -> Optional[papis.document.Document]:
         """Use the attribute `cls.parsed_search` to match the `doc` document
         to the previously parsed query.
         :param doc: Papis document to match against.
         :type  doc: papis.document.Document
-        :returns: True if it matches, False if some query requirement does
-            not match.
 
         >>> import papis.document
         >>> from papis.database.cache import match_document
@@ -44,8 +47,8 @@ class DocMatcher(object):
         ([(['heisenberg'], {})], {})
         >>> DocMatcher.return_if_match(doc) is not None
         False
-        >>> DocMatcher.parse('title = ein')
-        ([(['title', '=', 'ein'], {})], {})
+        >>> DocMatcher.parse('title : ein')
+        ([(['title', ':', 'ein'], {})], {})
         >>> DocMatcher.return_if_match(doc) is not None
         True
 
@@ -58,13 +61,15 @@ class DocMatcher(object):
             elif len(parsed) == 3:
                 search = parsed[2]
                 sformat = cls.doc_format.replace('DOC_KEY', parsed[0])
-            match = doc if cls.matcher(doc, search, sformat) else None
+
+            if cls.matcher is not None:
+                match = doc if cls.matcher(doc, search, sformat) else None
             if not match:
                 break
         return match
 
     @classmethod
-    def set_search(cls, search):
+    def set_search(cls, search: str) -> None:
         """
         >>> DocMatcher.set_search('author = Hummel')
         >>> DocMatcher.search
@@ -73,7 +78,7 @@ class DocMatcher(object):
         cls.search = search
 
     @classmethod
-    def set_matcher(cls, matcher):
+    def set_matcher(cls, matcher: MATCHER_TYPE) -> None:
         """
         >>> from papis.database.cache import match_document
         >>> DocMatcher.set_matcher(match_document)
@@ -81,7 +86,7 @@ class DocMatcher(object):
         cls.matcher = matcher
 
     @classmethod
-    def parse(cls, search=None):
+    def parse(cls, search: Optional[str] = None) -> List[List[str]]:
         """Parse the main query text. This method will also set the
         class attribute `parsed_search` to the parsed query, and it will
         return it too.
@@ -92,14 +97,14 @@ class DocMatcher(object):
         :type  search: str
         :returns: Parsed query
         :rtype:  list
-        >>> print(DocMatcher.parse('hello author = einstein'))
-        [['hello'], ['author', '=', 'einstein']]
+        >>> print(DocMatcher.parse('hello author : einstein'))
+        [['hello'], ['author', ':', 'einstein']]
         >>> print(DocMatcher.parse(''))
         []
         >>> print(\
             DocMatcher.parse(\
-                '"hello world whatever =" tags = \\\'hello ====\\\''))
-        [['hello world whatever ='], ['tags', '=', 'hello ====']]
+                '"hello world whatever :" tags : \\\'hello ::::\\\''))
+        [['hello world whatever :'], ['tags', ':', 'hello ::::']]
         >>> print(DocMatcher.parse('hello'))
         [['hello']]
         """
@@ -109,31 +114,34 @@ class DocMatcher(object):
         return cls.parsed_search
 
 
-def parse_query(query_string):
-    """
-    >>> print(parse_query('hello   author = einstein'))
-    [['hello'], ['author', '=', 'einstein']]
-    """
+def parse_query(query_string: str) -> List[List[str]]:
     import pyparsing
     logger = logging.getLogger('query_parser')
     logger.debug('Parsing search')
-    papis_key = pyparsing.Word(pyparsing.alphanums + '-')
+
+    papis_key_word = pyparsing.Word(pyparsing.alphanums + '-._/')
+    papis_value_word = pyparsing.Word(pyparsing.alphanums + '-._/()')
+
     papis_value = pyparsing.QuotedString(
         quoteChar='"', escChar='\\', escQuote='\\'
     ) ^ pyparsing.QuotedString(
         quoteChar="'", escChar='\\', escQuote='\\'
-    ) ^ papis_key
-    equal = pyparsing.ZeroOrMore(" ") + \
-        pyparsing.Literal('=') + \
+    ) ^ papis_value_word
+
+    equal = (
+        pyparsing.ZeroOrMore(" ") +
+        pyparsing.Literal(':') +
         pyparsing.ZeroOrMore(" ")
+    )
 
     papis_query = pyparsing.ZeroOrMore(
         pyparsing.Group(
             pyparsing.ZeroOrMore(
-                papis_key + equal
+                papis_key_word + equal
             ) + papis_value
         )
     )
-    parsed = papis_query.parseString(query_string)
+    parsed = papis_query.parseString(query_string)  # type: List[List[str]]
+
     logger.debug('Parsed query = %s' % parsed)
     return parsed
